#ifndef _STIM_H_
#define _STIM_H_

#include <iomanip>
using std::setw;
#include "adder.h"

// WIDTH is defined in adder.h
const int DEPTH = (1 << WIDTH);

SC_MODULE(stim) {

  sc_in_clk i_clk;
  sc_out<bool> o_rst;
  sc_out<sc_uint<WIDTH> > o_a;
  sc_out<sc_uint<WIDTH> > o_b;
  sc_in<sc_uint<WIDTH+1> > i_sum;

  sc_uint<WIDTH> t_a;
  sc_uint<WIDTH> t_b;
  sc_uint<WIDTH+1> t_sum;

  void stim_gen() {
    for (int a = 0; a < DEPTH; a++) {
      for (int b = 0; b < DEPTH; b++) {
        wait();
        o_a.write(a);
        o_b.write(b);
      }
    }
  }

  void sink() {
    while(true) {
      t_sum=i_sum.read();
      wait();
    }
  }

  void monitor() {
    cout << setw(12) << "time" << setw(12) << "a" << setw(12) << "b" 
         << setw(12) << "sum"  << endl;
    while (true) {
      wait();
      t_a=o_a.read();
      t_b=o_b.read();
      t_sum=i_sum.read();
      cout << setw(12) << sc_time_stamp();
      cout << setw(12) << t_a.to_string(SC_BIN);
      cout << setw(12) << t_b.to_string(SC_BIN);
      cout << setw(12) << t_sum.to_string(SC_BIN) << endl;
      cout << "===============================================================================================" << endl;
    }
  }

  SC_CTOR(stim) {

    SC_THREAD(stim_gen);
    sensitive << i_clk.pos();
    dont_initialize();

    SC_THREAD(sink);
    sensitive << i_clk.pos();

    // the first row of output table
    SC_THREAD(monitor);
    sensitive << i_clk.pos();
 
  }
};

#endif
